import {
  HttpStatus,
  Inject,
  Injectable,
  UnauthorizedException,
} from '@nestjs/common';
import { validatePassword } from 'src/core/helpers/password/bcrypt.helpers';
import { BaseCustomManager } from 'src/core/modules/domain/usecase/managers/base-custom.manager';
import { SessionService } from 'src/core/sessions';
import { EventTopics } from 'src/core/strings/constants/interface.constants';
import { UserModel } from 'src/modules/user-related/user/data/models/user.model';
import { UserEntity } from 'src/modules/user-related/user/domain/entities/user.entity';

@Injectable()
export class LoginManager extends BaseCustomManager<UserEntity> {
  @Inject()
  protected session: SessionService;
  protected token;
  protected userLogin;

  async validateProcess(): Promise<void> {
    return;
  }

  async beforeProcess(): Promise<void> {
    return;
  }

  async process(): Promise<void> {
    this.userLogin = await this.dataService.getOneByOptions({
      where: {
        username: this.data.username,
      },
    });
    if (!this.userLogin) this.throwError();

    const valid = await validatePassword(
      this.data.password,
      this.userLogin?.password,
    );
    if (!valid) this.throwError();

    const tokenData = {
      id: this.userLogin.id,
      name: this.userLogin.name,
      username: this.userLogin.username,
      role: this.userLogin.role,
      user_privilege_id: this.userLogin.user_privilege_id,
    };

    this.token = this.session.createAccessToken(tokenData);
    const refreshToken = this.session.createAccessToken(tokenData);

    await this.dataService.update(
      this.queryRunner,
      this.entityTarget,
      { id: this.userLogin.id },
      {
        refresh_token: refreshToken,
      },
    );

    return;
  }

  async afterProcess(): Promise<void> {
    return;
  }

  getResult() {
    return {
      id: this.userLogin.id,
      name: this.userLogin.name,
      username: this.userLogin.username,
      role: this.userLogin.role,
      user_privilege_id: this.userLogin.user_privilege_id,
      token: this.token,
    };
  }

  get entityTarget(): any {
    return UserModel;
  }

  get eventTopics(): EventTopics[] {
    return [];
  }

  throwError() {
    throw new UnauthorizedException({
      statusCode: HttpStatus.UNAUTHORIZED,
      message: `Failed! You have entered an invalid username or password`,
      error: 'Unauthorized',
    });
  }
}
